#!/bin/bash
# ipchains.mod.sh
# A dulog module
# Tries to represent "input REJECT" ipchains logs in a sane way. 
# by Icon Riabitsev <icon@phy.duke.edu>
#
# $Id: ipchains.mod.sh,v 1.5 2001/08/15 17:38:19 icon Exp $
#

spew(){
 if [ -z "$QUIET" ]; then
  echo -en "$1"
 fi
}

getproto() {
 # $1 should be protocol digit. e.g. 6 for tcp
 # sets $PROTONAME
 unset PROTONAME
 PROTONAME=`egrep -v "^#" /etc/protocols | egrep "^.*[[:space:]]$1[[:space:]]" | awk '{print $1}'`
 if [ ! -n $PROTONAME ]; then
   return 0
 else
   return 1
 fi
}

getservicename() {
 # $1 should be the port.
 # $2 should be protocol name, e.g. "tcp" or "udp".
 # Returns 0 and $SERVICENAME if port resolves to service or trojan
 # Returns 1 if port doesn't resolve.

 unset SERVICENAME
 # first grep /etc/services
 local SERVICELINE=`egrep "[[:space:]]$1/$2" /etc/services`
 if [ -z "$SERVICELINE" ]; then
   # nothing. Try trojans.
   TROJANLIST=$CONFDIR/trojans.list
   local TROJANLINE=`egrep "$2:$1:" $TROJANLIST`
   if [ -z "$TROJANLINE" ]; then
    # nothing.
    return 1
   else
    SERVICENAME=`echo -n "$1 TROJAN - ${TROJANLINE##*:}"`;
   fi
 else
   # a hit
   SERVICENAME=`echo $SERVICELINE | awk '{print $1}'`
 fi
 # make sure $SERVICENAME has no "/"'s.
 SERVICENAME=`echo $SERVICENAME | sed "s!/!|!g"`
 return 0
}  

gethostname() {
 # $1 should be the ip address
 unset HOSTDNS
 # head -n1 because some nimrods have multiple reverse-DNS settings
 local HOSTINFO=`host $1 2>/dev/null | grep "domain name pointer" | head -n1`
 if [ -z "$HOSTINFO" ]; then
   return 1
 else
   HOSTDNS=${HOSTINFO##* }
   HOSTDNS=${HOSTDNS%.}
   return 0
 fi
}

WORKPREFIX="$TMPPREFIX.portscan"

grep "input REJECT" $LOGCAT | sort | uniq > "$WORKPREFIX.step.0"

if [ -s $WORKPREFIX.step.0 ]; then
 # TODO: use getport(), silly.
 sed "s|^.* \([^[:space:]]*\) kernel:.* PROTO=\([[:digit:]]*\).*\(:.*\) L.*$|\1\3:\2|g; s/:6$/:tcp/g; s/:17$/:udp/g; s|.*/||g" "$WORKPREFIX.step.0" > "$WORKPREFIX.step.1"

 sort "$WORKPREFIX.step.1" | uniq -c > "$WORKPREFIX.hosts"

 sed "s/.* PROTO=\([[:digit:]]*\) \(.*\):[[:digit:]]* .*:\([[:digit:]]*\) L=.*/\2:\3:\1/g" "$WORKPREFIX.step.0" | sort | uniq > "$WORKPREFIX.step.3"
 spew "Processing offenders"
 for OFFENDER in `sed "s/:.*:.*$//g" $WORKPREFIX.step.3 | sort | uniq`; do
  # get the bastard's host name
  if ! gethostname $OFFENDER; then
   # no host name
   HOSTDNS=$OFFENDER
  fi
  echo -n "$HOSTDNS ->" > "$WORKPREFIX.step.4"
  for ENTRY in `grep $OFFENDER $WORKPREFIX.step.3 | sed "s/$OFFENDER://g; s/:6$/:tcp/g; s/:17$/:udp/g"`; do
    echo -n " $ENTRY" >> $WORKPREFIX.step.4
  done
  echo >> $WORKPREFIX.step.4
 cat $WORKPREFIX.step.4 >> $WORKPREFIX.offenders
 spew "."
 done
 spew "done\n"
 
 # concat into a report
 echo -e "\nIPCHAINS REPORT\n-----------------\n::Scanned hosts:: \n(Attempts host:port)" > $WORKPREFIX.report
 cat $WORKPREFIX.hosts >> $WORKPREFIX.report
 echo -e "\n::Offenders::" >> $WORKPREFIX.report
 cat $WORKPREFIX.offenders >> $WORKPREFIX.report
 # replace

 for ENTRY in `uniq $WORKPREFIX.step.1 | sed 's/^.*:\(.*:.*\)$/\1/g' | sort| uniq`; do
  PORT=${ENTRY%:*}
  PROTO=${ENTRY#*:}
  if getservicename $PORT $PROTO; then
   sed "s/$PORT:$PROTO/$SERVICENAME/g" "$WORKPREFIX.report" > "$WORKPREFIX.report.new"
   mv "$WORKPREFIX.report.new" "$WORKPREFIX.report"
  fi
 done
 cat $WORKPREFIX.report >> $LOGREPORT
 if [ ! -z "$WEED" ]; then
  grep -v "input REJECT" $LOGCAT > $LOGCAT.new
  mv $LOGCAT.new $LOGCAT
 fi
fi
